/****************************************************************************
** Meta object code from reading C++ file 'Session.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "Session.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Session.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Konsole7SessionE_t {};
} // unnamed namespace

template <> constexpr inline auto Konsole::Session::qt_create_metaobjectdata<qt_meta_tag_ZN7Konsole7SessionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Konsole::Session",
        "started",
        "",
        "almostFinished",
        "finished",
        "receivedData",
        "text",
        "titleChanged",
        "profileChanged",
        "profile",
        "stateChanged",
        "state",
        "currentDirectoryChanged",
        "dir",
        "bellRequest",
        "message",
        "changeTabTextColorRequest",
        "changeBackgroundColorRequest",
        "QColor",
        "openUrlRequest",
        "url",
        "resizeRequest",
        "QSize",
        "size",
        "profileChangeCommandReceived",
        "flowControlEnabledChanged",
        "enabled",
        "primaryScreenInUse",
        "use",
        "cursorChanged",
        "Emulation::KeyboardCursorShape",
        "cursorShape",
        "blinkingCursorEnabled",
        "silence",
        "activity",
        "sessionUninstallTerminal",
        "commandname",
        "titleArgsChange",
        "key",
        "value",
        "shellWarningMessage",
        "currentShell",
        "isSuccess",
        "osc52ClipboardRequest",
        "target",
        "base64Data",
        "run",
        "runEmptyPTY",
        "close",
        "setUserTitle",
        "caption",
        "done",
        "onReceiveBlock",
        "const char*",
        "buffer",
        "len",
        "isCommandExec",
        "monitorTimerDone",
        "onViewSizeChange",
        "height",
        "width",
        "onEmulationSizeChange",
        "activityStateSet",
        "viewDestroyed",
        "view",
        "onUpdateTitleArgs",
        "onPrimaryScreenInUse",
        "name",
        "processId",
        "keyBindings"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'almostFinished'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'receivedData'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'profileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'currentDirectoryChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'bellRequest'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'changeTabTextColorRequest'
        QtMocHelpers::SignalData<void(int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'changeBackgroundColorRequest'
        QtMocHelpers::SignalData<void(const QColor &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Signal 'openUrlRequest'
        QtMocHelpers::SignalData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Signal 'resizeRequest'
        QtMocHelpers::SignalData<void(const QSize &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'profileChangeCommandReceived'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'flowControlEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Signal 'primaryScreenInUse'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Signal 'cursorChanged'
        QtMocHelpers::SignalData<void(Emulation::KeyboardCursorShape, bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { QMetaType::Bool, 32 },
        }}),
        // Signal 'silence'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activity'
        QtMocHelpers::SignalData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sessionUninstallTerminal'
        QtMocHelpers::SignalData<bool(QString)>(35, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 36 },
        }}),
        // Signal 'titleArgsChange'
        QtMocHelpers::SignalData<void(QString, QString)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 38 }, { QMetaType::QString, 39 },
        }}),
        // Signal 'shellWarningMessage'
        QtMocHelpers::SignalData<void(QString, bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 41 }, { QMetaType::Bool, 42 },
        }}),
        // Signal 'osc52ClipboardRequest'
        QtMocHelpers::SignalData<void(char, const QString &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Char, 44 }, { QMetaType::QString, 45 },
        }}),
        // Slot 'run'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'runEmptyPTY'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setUserTitle'
        QtMocHelpers::SlotData<void(int, const QString &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::QString, 50 },
        }}),
        // Slot 'done'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'onReceiveBlock'
        QtMocHelpers::SlotData<void(const char *, int, bool)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 53, 54 }, { QMetaType::Int, 55 }, { QMetaType::Bool, 56 },
        }}),
        // Slot 'monitorTimerDone'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onViewSizeChange'
        QtMocHelpers::SlotData<void(int, int)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 59 }, { QMetaType::Int, 60 },
        }}),
        // Slot 'onEmulationSizeChange'
        QtMocHelpers::SlotData<void(QSize)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Slot 'activityStateSet'
        QtMocHelpers::SlotData<void(int)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'viewDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 64 },
        }}),
        // Slot 'onUpdateTitleArgs'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPrimaryScreenInUse'
        QtMocHelpers::SlotData<void(bool)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(67, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'processId'
        QtMocHelpers::PropertyData<int>(68, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'keyBindings'
        QtMocHelpers::PropertyData<QString>(69, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'size'
        QtMocHelpers::PropertyData<QSize>(23, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Session, qt_meta_tag_ZN7Konsole7SessionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Konsole::Session::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Konsole7SessionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Konsole7SessionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Konsole7SessionE_t>.metaTypes,
    nullptr
} };

void Konsole::Session::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Session *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->almostFinished(); break;
        case 2: _t->finished(); break;
        case 3: _t->receivedData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->titleChanged(); break;
        case 5: _t->profileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->currentDirectoryChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->bellRequest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->changeTabTextColorRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->changeBackgroundColorRequest((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 11: _t->openUrlRequest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->resizeRequest((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 13: _t->profileChangeCommandReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->flowControlEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->primaryScreenInUse((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->cursorChanged((*reinterpret_cast<std::add_pointer_t<Emulation::KeyboardCursorShape>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->silence(); break;
        case 18: _t->activity(); break;
        case 19: { bool _r = _t->sessionUninstallTerminal((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->titleArgsChange((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 21: _t->shellWarningMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 22: _t->osc52ClipboardRequest((*reinterpret_cast<std::add_pointer_t<char>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 23: _t->run(); break;
        case 24: _t->runEmptyPTY(); break;
        case 25: _t->close(); break;
        case 26: _t->setUserTitle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 27: _t->done((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->onReceiveBlock((*reinterpret_cast<std::add_pointer_t<const char*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 29: _t->monitorTimerDone(); break;
        case 30: _t->onViewSizeChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 31: _t->onEmulationSizeChange((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 32: _t->activityStateSet((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 33: _t->viewDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 34: _t->onUpdateTitleArgs(); break;
        case 35: _t->onPrimaryScreenInUse((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::almostFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QString & )>(_a, &Session::receivedData, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::titleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QString & )>(_a, &Session::profileChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(int )>(_a, &Session::stateChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QString & )>(_a, &Session::currentDirectoryChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QString & )>(_a, &Session::bellRequest, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(int )>(_a, &Session::changeTabTextColorRequest, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QColor & )>(_a, &Session::changeBackgroundColorRequest, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QString & )>(_a, &Session::openUrlRequest, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QSize & )>(_a, &Session::resizeRequest, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(const QString & )>(_a, &Session::profileChangeCommandReceived, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(bool )>(_a, &Session::flowControlEnabledChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(bool )>(_a, &Session::primaryScreenInUse, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(Emulation::KeyboardCursorShape , bool )>(_a, &Session::cursorChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::silence, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::activity, 18))
            return;
        if (QtMocHelpers::indexOfMethod<bool (Session::*)(QString )>(_a, &Session::sessionUninstallTerminal, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(QString , QString )>(_a, &Session::titleArgsChange, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(QString , bool )>(_a, &Session::shellWarningMessage, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(char , const QString & )>(_a, &Session::osc52ClipboardRequest, 22))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->nameTitle(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->processId(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->keyBindings(); break;
        case 3: *reinterpret_cast<QSize*>(_v) = _t->size(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setKeyBindings(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setSize(*reinterpret_cast<QSize*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Konsole::Session::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Konsole::Session::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Konsole7SessionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Konsole::Session::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Konsole::Session::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Konsole::Session::almostFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Konsole::Session::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Konsole::Session::receivedData(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Konsole::Session::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Konsole::Session::profileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Konsole::Session::stateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Konsole::Session::currentDirectoryChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Konsole::Session::bellRequest(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Konsole::Session::changeTabTextColorRequest(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Konsole::Session::changeBackgroundColorRequest(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Konsole::Session::openUrlRequest(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Konsole::Session::resizeRequest(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Konsole::Session::profileChangeCommandReceived(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Konsole::Session::flowControlEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Konsole::Session::primaryScreenInUse(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Konsole::Session::cursorChanged(Emulation::KeyboardCursorShape _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void Konsole::Session::silence()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void Konsole::Session::activity()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
bool Konsole::Session::sessionUninstallTerminal(QString _t1)
{
    bool _t0{};
    QMetaObject::activate<bool>(this, &staticMetaObject, 19, std::addressof(_t0), _t1);
    return _t0;
}

// SIGNAL 20
void Konsole::Session::titleArgsChange(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1, _t2);
}

// SIGNAL 21
void Konsole::Session::shellWarningMessage(QString _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}

// SIGNAL 22
void Konsole::Session::osc52ClipboardRequest(char _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN7Konsole12SessionGroupE_t {};
} // unnamed namespace

template <> constexpr inline auto Konsole::SessionGroup::qt_create_metaobjectdata<qt_meta_tag_ZN7Konsole12SessionGroupE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Konsole::SessionGroup"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SessionGroup, qt_meta_tag_ZN7Konsole12SessionGroupE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Konsole::SessionGroup::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Konsole12SessionGroupE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Konsole12SessionGroupE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Konsole12SessionGroupE_t>.metaTypes,
    nullptr
} };

void Konsole::SessionGroup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SessionGroup *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Konsole::SessionGroup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Konsole::SessionGroup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Konsole12SessionGroupE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Konsole::SessionGroup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
