/****************************************************************************
** Meta object code from reading C++ file 'qtermwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "qtermwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtermwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QTermWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto QTermWidget::qt_create_metaobjectdata<qt_meta_tag_ZN11QTermWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QTermWidget",
        "finished",
        "",
        "copyAvailable",
        "termGetFocus",
        "termLostFocus",
        "leftMouseClick",
        "termKeyPressed",
        "QKeyEvent*",
        "urlActivated",
        "QUrl",
        "fromContextMenu",
        "bell",
        "message",
        "activity",
        "silence",
        "uninstallTerminal",
        "commandname",
        "processStarted",
        "sendUrlsToTerm",
        "const char*",
        "sendData",
        "const QTextCodec*",
        "profileChanged",
        "profile",
        "titleChanged",
        "shellWarningMessage",
        "currentShell",
        "isSuccess",
        "receivedData",
        "text",
        "isTermIdle",
        "bIdle",
        "sig_noMatchFound",
        "sig_matchFound",
        "titleArgsChange",
        "key",
        "value",
        "osc52ClipboardRequest",
        "target",
        "base64Data",
        "copyClipboard",
        "pasteClipboard",
        "pasteSelection",
        "zoomIn",
        "zoomOut",
        "setSize",
        "QSize",
        "setKeyBindings",
        "kb",
        "clear",
        "toggleShowSearchBar",
        "search",
        "txt",
        "forwards",
        "next",
        "clearSelection",
        "noMatchFound",
        "saveHistory",
        "QIODevice*",
        "device",
        "sessionFinished",
        "selectionChanged",
        "textSelected",
        "onChangedCursorPosition",
        "count",
        "matchFound",
        "startColumn",
        "startLine",
        "endColumn",
        "endLine",
        "lastBackwardsPosition",
        "loseChinese",
        "matchChinese",
        "cursorChanged",
        "Konsole::Emulation::KeyboardCursorShape",
        "cursorShape",
        "blinkingCursorEnabled",
        "snapshot"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'copyAvailable'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'termGetFocus'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'termLostFocus'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leftMouseClick'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'termKeyPressed'
        QtMocHelpers::SignalData<void(QKeyEvent *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'urlActivated'
        QtMocHelpers::SignalData<void(const QUrl &, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'bell'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'activity'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'silence'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'uninstallTerminal'
        QtMocHelpers::SignalData<bool(QString)>(16, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'processStarted'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sendUrlsToTerm'
        QtMocHelpers::SignalData<void(const char *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 },
        }}),
        // Signal 'sendData'
        QtMocHelpers::SignalData<void(const char *, int, const QTextCodec *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 }, { QMetaType::Int, 2 }, { 0x80000000 | 22, 2 },
        }}),
        // Signal 'profileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shellWarningMessage'
        QtMocHelpers::SignalData<void(QString, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 27 }, { QMetaType::Bool, 28 },
        }}),
        // Signal 'receivedData'
        QtMocHelpers::SignalData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Signal 'isTermIdle'
        QtMocHelpers::SignalData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Signal 'sig_noMatchFound'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sig_matchFound'
        QtMocHelpers::SignalData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleArgsChange'
        QtMocHelpers::SignalData<void(QString, QString)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { QMetaType::QString, 37 },
        }}),
        // Signal 'osc52ClipboardRequest'
        QtMocHelpers::SignalData<void(char, const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Char, 39 }, { QMetaType::QString, 40 },
        }}),
        // Slot 'copyClipboard'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pasteClipboard'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pasteSelection'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSize'
        QtMocHelpers::SlotData<void(const QSize &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 2 },
        }}),
        // Slot 'setKeyBindings'
        QtMocHelpers::SlotData<void(const QString &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
        // Slot 'clear'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleShowSearchBar'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'search'
        QtMocHelpers::SlotData<void(QString, bool, bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 53 }, { QMetaType::Bool, 54 }, { QMetaType::Bool, 55 },
        }}),
        // Slot 'clearSelection'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'noMatchFound'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveHistory'
        QtMocHelpers::SlotData<void(QIODevice *)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'sessionFinished'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'onChangedCursorPosition'
        QtMocHelpers::SlotData<void(int)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 65 },
        }}),
        // Slot 'matchFound'
        QtMocHelpers::SlotData<void(int, int, int, int, int, int, int)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 67 }, { QMetaType::Int, 68 }, { QMetaType::Int, 69 }, { QMetaType::Int, 70 },
            { QMetaType::Int, 71 }, { QMetaType::Int, 72 }, { QMetaType::Int, 73 },
        }}),
        // Slot 'cursorChanged'
        QtMocHelpers::SlotData<void(Konsole::Emulation::KeyboardCursorShape, bool)>(74, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 75, 76 }, { QMetaType::Bool, 77 },
        }}),
        // Slot 'snapshot'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QTermWidget, qt_meta_tag_ZN11QTermWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QTermWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QTermWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QTermWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QTermWidgetE_t>.metaTypes,
    nullptr
} };

void QTermWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QTermWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished(); break;
        case 1: _t->copyAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->termGetFocus(); break;
        case 3: _t->termLostFocus(); break;
        case 4: _t->leftMouseClick(); break;
        case 5: _t->termKeyPressed((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 6: _t->urlActivated((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->bell((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->activity(); break;
        case 9: _t->silence(); break;
        case 10: { bool _r = _t->uninstallTerminal((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->processStarted(); break;
        case 12: _t->sendUrlsToTerm((*reinterpret_cast<std::add_pointer_t<const char*>>(_a[1]))); break;
        case 13: _t->sendData((*reinterpret_cast<std::add_pointer_t<const char*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<const QTextCodec*>>(_a[3]))); break;
        case 14: _t->profileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->titleChanged(); break;
        case 16: _t->shellWarningMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->receivedData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->isTermIdle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->sig_noMatchFound(); break;
        case 20: _t->sig_matchFound(); break;
        case 21: _t->titleArgsChange((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 22: _t->osc52ClipboardRequest((*reinterpret_cast<std::add_pointer_t<char>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 23: _t->copyClipboard(); break;
        case 24: _t->pasteClipboard(); break;
        case 25: _t->pasteSelection(); break;
        case 26: _t->zoomIn(); break;
        case 27: _t->zoomOut(); break;
        case 28: _t->setSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 29: _t->setKeyBindings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->clear(); break;
        case 31: _t->toggleShowSearchBar(); break;
        case 32: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 33: _t->clearSelection(); break;
        case 34: _t->noMatchFound(); break;
        case 35: _t->saveHistory((*reinterpret_cast<std::add_pointer_t<QIODevice*>>(_a[1]))); break;
        case 36: _t->sessionFinished(); break;
        case 37: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->onChangedCursorPosition((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->matchFound((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[7]))); break;
        case 40: _t->cursorChanged((*reinterpret_cast<std::add_pointer_t<Konsole::Emulation::KeyboardCursorShape>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 41: _t->snapshot(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(bool )>(_a, &QTermWidget::copyAvailable, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::termGetFocus, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::termLostFocus, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::leftMouseClick, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(QKeyEvent * )>(_a, &QTermWidget::termKeyPressed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(const QUrl & , bool )>(_a, &QTermWidget::urlActivated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(const QString & )>(_a, &QTermWidget::bell, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::activity, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::silence, 9))
            return;
        if (QtMocHelpers::indexOfMethod<bool (QTermWidget::*)(QString )>(_a, &QTermWidget::uninstallTerminal, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::processStarted, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(const char * )>(_a, &QTermWidget::sendUrlsToTerm, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(const char * , int , const QTextCodec * )>(_a, &QTermWidget::sendData, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(const QString & )>(_a, &QTermWidget::profileChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::titleChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(QString , bool )>(_a, &QTermWidget::shellWarningMessage, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(const QString & )>(_a, &QTermWidget::receivedData, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(bool )>(_a, &QTermWidget::isTermIdle, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::sig_noMatchFound, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)()>(_a, &QTermWidget::sig_matchFound, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(QString , QString )>(_a, &QTermWidget::titleArgsChange, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTermWidget::*)(char , const QString & )>(_a, &QTermWidget::osc52ClipboardRequest, 22))
            return;
    }
}

const QMetaObject *QTermWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QTermWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QTermWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int QTermWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 42;
    }
    return _id;
}

// SIGNAL 0
void QTermWidget::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QTermWidget::copyAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QTermWidget::termGetFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QTermWidget::termLostFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QTermWidget::leftMouseClick()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QTermWidget::termKeyPressed(QKeyEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QTermWidget::urlActivated(const QUrl & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void QTermWidget::bell(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QTermWidget::activity()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QTermWidget::silence()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
bool QTermWidget::uninstallTerminal(QString _t1)
{
    bool _t0{};
    QMetaObject::activate<bool>(this, &staticMetaObject, 10, std::addressof(_t0), _t1);
    return _t0;
}

// SIGNAL 11
void QTermWidget::processStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QTermWidget::sendUrlsToTerm(const char * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QTermWidget::sendData(const char * _t1, int _t2, const QTextCodec * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2, _t3);
}

// SIGNAL 14
void QTermWidget::profileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QTermWidget::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QTermWidget::shellWarningMessage(QString _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void QTermWidget::receivedData(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QTermWidget::isTermIdle(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QTermWidget::sig_noMatchFound()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QTermWidget::sig_matchFound()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QTermWidget::titleArgsChange(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}

// SIGNAL 22
void QTermWidget::osc52ClipboardRequest(char _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}
QT_WARNING_POP
